#!/usr/local/bin/perl -w

# Perl pragma to restrict unsafe constructs
use strict;

# Issue warnings about suspicious programming.
use warnings;

use lib "/usr/people/jmrc/Projects/SVN/MOBY/Exception/";
use MobyException;


print "\n1. Test of ERROR exception reporting ------------------------------------------------------------\n";
my ($excep);
eval {
	local(*INFILE);
	open(INFILE, "/usr/local/jm/toma") || die ($excep = MobyException->new(
								code => 200, 
								queryID => 69,
								refElement => 'test',
								message => 'Add new description',
								type => 'Error',
						)); 
};
if ($@) {
	# Moby Exception
	if ($excep->isa('MobyException')) {

		print "Message: ".$excep->getExceptionMessage()."\n";
		print "Code: ".$excep->getExceptionCode()."\n";
		print "Type: ".$excep->getExceptionType()."\n";

		print "Exception Response:\n".$excep->retrieveExceptionResponse()."\n";
		print "Empty MobyData Response:\n".$excep->retrieveEmptyMobyData()."\n";
		
		my ($emptyMobyResponse) = "\n\t".$excep->retrieveEmptyMobySimple('outputArticle')."\n";
		($emptyMobyResponse) .= "\n\t".$excep->retrieveEmptyMobyCollection('outputArticle2')."\n";
		print "Moby Response: \n".$excep->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
	}
}


print "\n2. Test of WARNING exception reporting ------------------------------------------------------------\n";
my ($excep2);
eval {
	local(*INFILE);
	open(INFILE, "/usr/local/jm/toma") || die ($excep2 = MobyException->new(
								code => 200, 
								queryID => 69,
								refElement => 'test',
								message => 'Add new description',
								type => 'WARNing',
						)); 
};
if ($@) {
	# Moby Exception
	if ($excep2->isa('MobyException')) {

		print "Message: ".$excep2->getExceptionMessage()."\n";
		print "Code: ".$excep2->getExceptionCode()."\n";
		print "Type: ".$excep2->getExceptionType()."\n";

		print "Exception Response:\n".$excep2->retrieveExceptionResponse()."\n";
		print "Empty MobyData Response:\n".$excep2->retrieveEmptyMobyData()."\n";
		
		my ($emptyMobyResponse) = "\n\t".$excep2->retrieveEmptyMobySimple('outputArticle')."\n";
		($emptyMobyResponse) .= "\n\t".$excep2->retrieveEmptyMobyCollection('outputArticle2')."\n";
		print "Moby Response: \n".$excep2->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
	}
}

print "\n3. Test of INFORMATION exception reporting ------------------------------------------------------------\n";
my ($excep3);
eval {
	local(*INFILE);
	open(INFILE, "/usr/local/jm/toma") || die ($excep3 = MobyException->new(
								code => 200, 
								queryID => 69,
								refElement => 'test',
								message => 'Add new description',
								type => 'information',
						)); 
};
if ($@) {
	# Moby Exception
	if ($excep3->isa('MobyException')) {
		print "Message: ".$excep3->getExceptionMessage()."\n";
		print "Code: ".$excep3->getExceptionCode()."\n";
		print "Type: ".$excep3->getExceptionType()."\n";

		print "Exception Response:\n".$excep3->retrieveExceptionResponse()."\n";
		print "Empty MobyData Response:\n".$excep3->retrieveEmptyMobyData()."\n";
		
		my ($emptyMobyResponse) = "\n\t".$excep3->retrieveEmptyMobySimple('outputArticle')."\n";
		($emptyMobyResponse) .= "\n\t".$excep3->retrieveEmptyMobyCollection('outputArticle2')."\n";
		print "Moby Response: \n".$excep3->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
	}
}

print "\n6. Test of new ways to 'bless' mobyException class-------------------------------------------------\n";
my ($excep6);
eval {
	local(*INFILE);
	open(INFILE, "/usr/local/jm/toma") || die ($excep6 = MobyException->new());
};
if ($@) {
	# Moby Exception
	if ($excep6->isa('MobyException')) {
		# Add new attributes
		$excep6->setExceptionCode(201);
		$excep6->setExceptionMessage("Add new description");
		$excep6->setExceptionMessage("No!! Better this one");
		$excep6->setExceptionType("ERROr");

		print "Message: ".$excep3->getExceptionMessage()."\n";
		print "Code: ".$excep3->getExceptionCode()."\n";
		print "Type: ".$excep3->getExceptionType()."\n";

		print "Exception Response:\n".$excep3->retrieveExceptionResponse()."\n";
		print "Empty MobyData Response:\n".$excep3->retrieveEmptyMobyData()."\n";
		
		my ($emptyMobyResponse) = "\n\t".$excep3->retrieveEmptyMobySimple('outputArticle')."\n";
		($emptyMobyResponse) .= "\n\t".$excep3->retrieveEmptyMobyCollection('outputArticle2')."\n";
		print "Moby Response: \n".$excep3->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
	}
}

print "\n7. Test of new ways to 'bless' mobyException class: out of eval ------------------------------------------\n";
my ($test) = 'error';
if ($test eq 'error') {
	my ($excep7) = MobyException->new(queryID => 69, refElement => 'test');

	# Add new attributes
	$excep7->setExceptionCode(201);
	$excep7->setExceptionMessage("Add new description");
	$excep7->setExceptionMessage("No!! Better this one");
	$excep7->setExceptionType("WarNING");

	print "Message: ".$excep7->getExceptionMessage()."\n";
	print "Code: ".$excep7->getExceptionCode()."\n";
	print "Type: ".$excep7->getExceptionType()."\n";

	print "Exception Response:\n".$excep7->retrieveExceptionResponse()."\n";
	print "Empty MobyData Response:\n".$excep7->retrieveEmptyMobyData()."\n";
		
	my ($emptyMobyResponse) = "\n\t".$excep7->retrieveEmptyMobySimple('outputArticle')."\n";
	($emptyMobyResponse) .= "\n\t".$excep7->retrieveEmptyMobyCollection('outputArticle2')."\n";
	print "Moby Response: \n".$excep7->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
}

print "Select one test where final result is error (8,9, 10): ";
my $errorTest=<STDIN>;
chomp($errorTest);
if ($errorTest == 8) {
	print "\n8. Test of CODE of exception: EXCEPTION CODE DOES NOT EXISTS ----------------------------------\n";
	my ($excep8);
	eval {
		local(*INFILE);
		open(INFILE, "/usr/local/jm/toma") || die ($excep8 = MobyException->new(
									code => 69,  # EXCEPTION CODE DOES NOT EXISTS
									queryID => 69,
									refElement => 'test',
									type => 'information',
							)); 
	};
	if ($@) {
		# Moby Exception
		if ($excep8->isa('MobyException')) {
			print "Message: ".$excep8->getExceptionMessage()."\n";
			print "Code: ".$excep8->getExceptionCode()."\n";
			print "Type: ".$excep8->getExceptionType()."\n";
			print "Exception Response:\n".$excep8->retrieveExceptionResponse()."\n";
			print "Empty MobyData Response:\n".$excep8->retrieveEmptyMobyData()."\n";
			
			my ($emptyMobyResponse) = "\n\t".$excep8->retrieveEmptyMobySimple('outputArticle')."\n";
			($emptyMobyResponse) .= "\n\t".$excep8->retrieveEmptyMobyCollection('outputArticle2')."\n";
			print "Moby Response: \n".$excep8->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
		}
	}
} elsif ($errorTest == 9) {

	print "\n9. Test of type of exception: EXCEPTION TYPE DOES NOT EXISTS ----------------------------------\n";
	my ($excep9);
	eval {
		local(*INFILE);
		open(INFILE, "/usr/local/jm/toma") || die ($excep9 = MobyException->new(
									code => 201,  
									queryID => 69,
									refElement => 'test',
									type => 'errIr', # EXCEPTION TYPE DOES NOT EXISTS
							)); 
	};
	if ($@) {
		# Moby Exception
		if ($excep9->isa('MobyException')) {
			print "Message: ".$excep9->getExceptionMessage()."\n";
			print "Code: ".$excep9->getExceptionCode()."\n";
			print "Type: ".$excep9->getExceptionType()."\n";
			print "Exception Response:\n".$excep9->retrieveExceptionResponse()."\n";
			print "Empty MobyData Response:\n".$excep9->retrieveEmptyMobyData()."\n";
			
			my ($emptyMobyResponse) = "\n\t".$excep9->retrieveEmptyMobySimple('outputArticle')."\n";
			($emptyMobyResponse) .= "\n\t".$excep9->retrieveEmptyMobyCollection('outputArticle2')."\n";
			print "Moby Response: \n".$excep9->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
		}
	}
} elsif ($errorTest == 10) {

	print "\n10. Test of new ways to 'bless' mobyException class-------------------------------------------------\n";
	my ($excep10) = MobyException->new(queryID => 69, refElement => 'test');

	# Add new attributes
	$excep10->setExceptionCode(201);
	$excep10->setExceptionMessage("Add new description");
	$excep10->setExceptionMessage("No!! Better this one");
	$excep10->setExceptionType("ERROWWWr");

	print "Message: ".$excep10->getExceptionMessage()."\n";
	print "Code: ".$excep10->getExceptionCode()."\n";
	print "Type: ".$excep10->getExceptionType()."\n";

	print "Exception Response:\n".$excep10->retrieveExceptionResponse()."\n";
	print "Empty MobyData Response:\n".$excep10->retrieveEmptyMobyData()."\n";
		
	my ($emptyMobyResponse) = "\n\t".$excep10->retrieveEmptyMobySimple('outputArticle')."\n";
	($emptyMobyResponse) .= "\n\t".$excep10->retrieveEmptyMobyCollection('outputArticle2')."\n";
	print "Moby Response: \n".$excep10->embedMOBYArticlesIntoMOBYData($emptyMobyResponse)."\n";
}
